# Author: Oriol Filter
# Date: 06/04/2022


$DebugPreference = "Continue"
$ErrorActionPreference = "Stop"
$SCRIPT_PATH=$PSScriptRoot
$HYPERV_FILE = "$SCRIPT_PATH/hyperv_list.txt"
$HOSTS_FILE = "$SCRIPT_PATH/hosts.txt"
## Objects

class VirtualizationServer {
    [string]$url
    [Boolean] _FindHost ($vname) {
        $type=$this.GetType().BaseType
        Write-Host "Find Host not implemented type $type"
        return $false
    }
    [Boolean] FindHost ($vname) {
        return $this._FindHost($vname)
    }

}

class HypervisorServer: VirtualizationServer{
    [Boolean] _FindHost ($vname) {
        $result=$false
        try
        {
            Invoke-Command -ComputerName $this.url {
                Param ($name) Get-VM -Name $name
            } -ArgumentList $vname
            $result=$?
        }
        catch
        {
            Format ("{0}: {1}",$this.url,$Error[0]) | Write-Debug
        }

        return $result
    }
}
class VMWareServer: VirtualizationServer{}

class VirtualizerManager{
    [Array]$_connected_servers=@()
    [Hashtable]$_found_vm=@{}
    [Boolean] __log_in($url){
        $status_message=""
        $result=$false
        try {
            $result=Invoke-Command -ComputerName $url { $true }
            $status_message = "OK"
        }
        catch [System.Management.Automation.Remoting.PSRemotingTransportException] {
            $status_message = Format("Error '{0}'",$_.Exception.GetType().fullname)
        }
        finally {
            $message = [string]::Format(">> [{0}] {1}",$url,$status_message)
            Write-Debug $message
        }
        return $result
    }
    append_session($type,$url)
    {
        #        $Allowed_Types = @(
        #        VMWareServer;
        #        HypervisorServer
        #        )
        #$Allowed_Types
        $type.GetType().IsValueType
        $result = $this.__log_in($url)
        if ($result)
        {
            $server=$type::new()
            $server.url=$url
            $this._connected_servers+=$server
        }
    }
    [Array] connected_servers() {
        return $this._connected_servers
    }
    find_host($vname){
        $hostlist=$vname
        if ($hostlist -is [Array]){
            $hostlist=@($hostlist)
        }
        foreach ($_vname in $hostlist)
        {
            for ($num = 0 ; $num -le $this._connected_servers.Length -1 ; $num++){
                    $server= $this._connected_servers[$num]
                    $result = $server.FindHost($_vname)
                    [String]::Format(">> {0} found in {1}? {2}", $_vname, $server.url, $result) | Write-Host
            }
            }
        }

    }

    # Get hyperv's
    function Get-Hosts-From-File($filepath)
    {
    ## In a future use a yaml as a .conf (with the hyperv and vmware listed)
    $file_content = Get-Content "$filepath"
    $splitted_file_content = $file_content.Split()
    $host_array=@()
    foreach ($hostname in $splitted_file_content)
    {
    if ($hostname)
    {
    $host_array+=$hostname
    }
    }
    return $host_array
    }

    # Advanced params?
    # https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_functions_advanced_parameters?view=powershell-7.2
    function load_hyperv($manager) {
    $hyperv_url = Get-Hosts-From-File($HYPERV_FILE)
    foreach ($url in $hyperv_url) {
    $manager.append_session([HypervisorServer], $url)
    }
    }


    function Main{
    $manager = [VirtualizerManager]::new()
    load_hyperv($manager)
    $vmhosts= Get-Hosts-From-File($HOSTS_FILE)
    $manager.find_host($vmhosts)
}


Main
